package {
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.objects.parsers.DAE;

	[SWF(width=640, height=480, backgroundColor=0x0, frameRate=30)]

	public class Gihyo2_2 extends PV3DARApp {
		
		// ___________________________________________________________________ 3D vars
	
		private var rootNode  :DisplayObject3D;
	
		// ___________________________________________________________________ Car vars
	
		private var topSpeed  :Number = 0;
		private var topSteer  :Number = 0;
		private var speed     :Number = 0;
		private var steer     :Number = 0;
	
		// ___________________________________________________________________ Keyboard vars
	
		private var keyRight   :Boolean = false;
		private var keyLeft    :Boolean = false;
		private var keyForward :Boolean = false;
		private var keyReverse :Boolean = false;

		public function Gihyo2_2() {
			this.init('Data/camera_para.dat', 'Data/flarlogo.pat');
		}
		
		protected override function onInit():void {
			super.onInit();
			
			var car:DAE = new DAE();
			car.load('Focus.dae');
			car.scale = 3;
			car.rotationX = 90;
			car.rotationZ = -90;
			this._baseNode.addChild(car);
			this.rootNode = car;

			stage.addEventListener( KeyboardEvent.KEY_DOWN, keyDownHandler );
			stage.addEventListener( KeyboardEvent.KEY_UP, keyUpHandler );
	
			this.addEventListener( Event.ENTER_FRAME, loop3D );
		}


		// ___________________________________________________________________ Keyboard
	
		private function keyDownHandler( event :KeyboardEvent ):void
		{
			switch( event.keyCode )
			{
				case "W".charCodeAt():
				case Keyboard.UP:
					keyForward = true;
					keyReverse = false;
					break;
	
				case "S".charCodeAt():
				case Keyboard.DOWN:
					keyReverse = true;
					keyForward = false;
					break;
	
				case "A".charCodeAt():
				case Keyboard.LEFT:
					keyLeft = true;
					keyRight = false;
					break;
	
				case "D".charCodeAt():
				case Keyboard.RIGHT:
					keyRight = true;
					keyLeft = false;
					break;
			}
			//trace("keyDownHandler: " + event.keyCode);
		}
	
	
		private function keyUpHandler( event :KeyboardEvent ):void
		{
			switch( event.keyCode )
			{
				case "W".charCodeAt():
				case Keyboard.UP:
					keyForward = false;
					break;
	
				case "S".charCodeAt():
				case Keyboard.DOWN:
					keyReverse = false;
					break;
	
				case "A".charCodeAt():
				case Keyboard.LEFT:
					keyLeft = false;
					break;
	
				case "D".charCodeAt():
				case Keyboard.RIGHT:
					keyRight = false;
					break;
			}
			//trace("keyUpHandler: " + event.keyCode);
		}
	
	
		// ___________________________________________________________________ driveCar
	
		private function driveCar():void
		{
			// Speed
			if( keyForward )
			{
				topSpeed = 2;
			}
			else if( keyReverse )
			{
				topSpeed = -1;
			}
			else
			{
				topSpeed = 0;
			}
	
			speed -= ( speed - topSpeed ) / 10;
	
			// Steer
			if( keyRight )
			{
				if( topSteer < 45 )
				{
					topSteer += 5;
				}
			}
			else if( keyLeft )
			{
				if( topSteer > -45 )
				{
					topSteer -= 5;
				}
			}
			else
			{
				topSteer -= topSteer / 5;
			}
	
			steer -= ( steer - topSteer ) / 2;
		}
	
	
		// ___________________________________________________________________________________________ updateCar
	
		private function updateCar( car :DisplayObject3D ):void
		{
			// Steer front wheels
			var steerFR :DisplayObject3D = car.getChildByName( "Steer_FR" );
			var steerFL :DisplayObject3D = car.getChildByName( "Steer_FL" );
	
			steerFR.rotationY = steer;
			steerFL.rotationY = steer;
	
			// Rotate wheels
			var wheelFR :DisplayObject3D = steerFR.getChildByName( "Wheel_FR" );
			var wheelFL :DisplayObject3D = steerFL.getChildByName( "Wheel_FL" );
			var wheelRR :DisplayObject3D = car.getChildByName( "Wheel_RR" );
			var wheelRL :DisplayObject3D = car.getChildByName( "Wheel_RL" );
	
			var roll :Number = speed/2
			wheelFR.roll(  roll );
			wheelRR.roll(  roll );
			wheelFL.roll( -roll );
			wheelRL.roll( -roll );
	
			// Steer car
			car.yaw( -speed * steer / 10 );
	
			// Move car
			car.moveForward( speed );
		}
	
	
		// ___________________________________________________________________________________________ loop3D
	
		private function loop3D( event :Event ):void
		{
//			camera.z = -300 + scene.container.mouseX * 5;
//			camera.y = Math.max( 0, this.mouseY ) * 5;
	
			// Get plane from rootNode
			var car :DisplayObject3D = this.rootNode.getChildByName("Focus", true);
			
			// Check if car has been loaded
			if( car )
			{
//				// Get plane from rootNode, we obviously don't need to check if it has been loaded.
//				var plane :DisplayObject3D = this.rootNode.getChildByName( "Plane" );
//	
//				// Check if car hits plane and change color
//				if( car.hitTestObject( plane ) )
//					plane.material.fillColor = 0xFFFFFF;
//				else
//					plane.material.fillColor = 0x333333;
	
				// Calculate current steer and speed
				driveCar();
	
				// Update car model
				updateCar( car );
			}
	
			// Render the scene
//			this.scene.renderCamera( camera );
		}
	}
}